//Test Code for 1-Wire Barometric Sensor with ATMega48 and BMP085
//
//Copyright (C) 2012  Tobias Mueller mail (at) tobynet.de
//
//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
// any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
//

#define _CRT_SECURE_NO_WARNINGS 1
#include "ownet.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int readBMP085_4s(int portnum, uchar *SerialNum,uchar block, short *s) {
	int rt=FALSE;
	uchar send_block[30],lastcrc8;
	int send_cnt,  i, loop=0;
	
	owSerialNum(portnum,SerialNum,FALSE);
	for (loop = 0; loop < 5; loop ++) {
		if (owAccess(portnum)) {
			//copy block in scratchpad
			if (!owWriteByte(portnum,0x47+block))
				continue;
			msDelay(100);
			// access the device
			if (owAccess(portnum))  {
				// create a block to send that reads the temperature
				// read scratchpad command
				send_cnt = 0;
				send_block[send_cnt++] = 0xBE;
				// now add the read bytes for data bytes and crc8
				for (i = 0; i < 10; i++) send_block[send_cnt++] = 0xFF;
				// now send the block
				if (owBlock(portnum,FALSE,send_block,send_cnt)) {
					// initialize the CRC8 
					setcrc8(portnum,0);
					// perform the CRC8 on the last 8 bytes of packet
					for (i = send_cnt - 10; i < send_cnt; i++)
						lastcrc8 = docrc8(portnum,send_block[i]);
					// verify CRC8 is correct
					if (lastcrc8 == 0x00) {
						// calculate the high-res temperature
						for(i=0;i<4;i++) {
							s[i]= ((short*) (send_block+2))[i];
						}
						/*for(i=0;i<10;i++) {
							printf("%i ",send_block[i]);
						}*/
						//printf("\n");
						// success
						rt = TRUE;
						break;
					} else printf("CRC_ERROR\n");
				}
			}
		}
	}
	return rt;
}

int printBMP085_calib(int portnum, uchar *SerialNum) {
	short s[4];
	if (readBMP085_4s(portnum,SerialNum,0,s)) {
		printf("ac1 %i\nac2 %i\nac3 %i\nac4 %i\n",s[0],s[1],s[2],(unsigned short)s[3]);
	} 
	if (readBMP085_4s(portnum,SerialNum,1,s)) {
		printf("ac5 %i\nac6 %i\nb1 %i\nb2 %i\n",(unsigned short)s[0],(unsigned short)s[1],s[2],s[3]);
	}
	if (readBMP085_4s(portnum,SerialNum,2,s)) {
		printf("mb %i\nmc %i\nmd %i\n",s[0],s[1],s[2]);
	}
	return 1;
}




int readBMP085(int portnum, uchar *SerialNum, double *Press, double *Temp, uchar *stat,long delay) {
	int rt=FALSE;
	uchar send_block[30],lastcrc8;
	int send_cnt,  i, loop=0;
	long *p,*t;
	
	owSerialNum(portnum,SerialNum,FALSE);
	for (loop = 0; loop < 5; loop ++) {
		if (owAccess(portnum)) {
			// send the convert command and start power delivery
			if (!owWriteByte(portnum,0x44))
				continue;
			// sleep for Conversion
			msDelay(delay);
			// access the device
			if (owAccess(portnum))  {
				// create a block to send that reads the temperature
				// read scratchpad command
				send_cnt = 0;
				send_block[send_cnt++] = 0xBE;
				// now add the read bytes for data bytes and crc8
				for (i = 0; i < 10; i++) send_block[send_cnt++] = 0xFF;
				// now send the block
				if (owBlock(portnum,FALSE,send_block,send_cnt)) {
					// initialize the CRC8 
					setcrc8(portnum,0);
					// perform the CRC8 on the last 8 bytes of packet
					for (i = send_cnt - 10; i < send_cnt; i++)
						lastcrc8 = docrc8(portnum,send_block[i]);
					// verify CRC8 is correct
					if (lastcrc8 == 0x00) {
						// calculate the high-res temperature
						t=(long*) (send_block+2);
						p=(long*) (send_block+6);
						*Press=(double)(*p)/100.0;
						*Temp=(double)(*t)/10.0;
						// success
						*stat=send_block[1];
						rt = TRUE;
						if ((send_block[1]&8)!=0) {
							printf("Measurement in progress\n");
							msDelay(1000);
							continue;
						}
						break;
					} else printf("CRC_ERROR\n");
				}
			}
		}
	}
	return rt;
}

int writeBMP085(int portnum, uchar *SerialNum,uchar stat) {
	int rt=FALSE;
	uchar send_block[30];
	int send_cnt;
	
	owSerialNum(portnum,SerialNum,FALSE);
	if (owAccess(portnum)) {
		send_cnt = 0;
		send_block[send_cnt++] = 0x4E;					
		send_block[send_cnt++] = stat;		
		owBlock(portnum,FALSE,send_block,send_cnt);

	}
	return TRUE;
}



void test_normal_mode(int portnum, uchar *SNum) {
	double Press,Temp;	
	uchar stat;
	writeBMP085(portnum,SNum,0);
	readBMP085(portnum,SNum,&Press,&Temp,&stat,25);
	if ((stat&1)!=0) printf("Oversembling on, ");
	if ((stat&2)!=0) printf("Multimess, ");
	if ((stat&4)!=0) printf("Continuously, ");
	if ((stat&8)!=0) printf("Measurement in Progress, ");
	printf("%.2f hPa, %.2f %cC\n",Press,Temp,248);
	printBMP085_calib(portnum, SNum);
}

void test_oversambling_mode(int portnum, uchar *SNum) {
	double Press,Temp;	
	uchar stat;
	writeBMP085(portnum,SNum,1);
	readBMP085(portnum,SNum,&Press,&Temp,&stat,50);
	if ((stat&1)!=0) printf("Oversembling on, ");
	if ((stat&2)!=0) printf("Multimess, ");
	if ((stat&4)!=0) printf("Continuously, ");
	if ((stat&8)!=0) printf("Measurement in Progress, ");
	printf("%.2f hPa, %.2f %cC\n",Press,Temp,248);
	printBMP085_calib(portnum, SNum);
}

void test_multi_mode(int portnum, uchar *SNum) {
	double Press,Temp;	
	uchar stat;
	writeBMP085(portnum,SNum,3);
	readBMP085(portnum,SNum,&Press,&Temp,&stat,800);
	if ((stat&1)!=0) printf("Oversembling on, ");
	if ((stat&2)!=0) printf("Multimess, ");
	if ((stat&4)!=0) printf("Continuously, ");
	if ((stat&8)!=0) printf("Measurement in Progress, ");
	printf("%.2f hPa, %.2f %cC\n",Press,Temp,248);
	printBMP085_calib(portnum, SNum);
}


void test_continuous_mode(int portnum, uchar *SNum) {
	double Press,Temp;	
	uchar stat;
	int i;
	writeBMP085(portnum,SNum,7+3*16);
	for(i=0;i<2000;i++) {
		readBMP085(portnum,SNum,&Press,&Temp,&stat,0);
		if ((stat&1)!=0) printf("Oversembling on, ");
		if ((stat&2)!=0) printf("Multimess, ");
		if ((stat&4)!=0) printf("Continuously, ");
		if ((stat&8)!=0) printf("Measurement in Progress, ");
		printf("%.2f hPa, %.2f %cC  (%i)\n",Press+37.84,Temp,248,i);
		msDelay(1000*60);
		//printBMP085_calib(portnum, SNum);
	}
}



int main(int argc, char *argv[]) {
	uchar SNum[8];
	int rslt,cnt,i;
	int portnum;
	char adapter[20];
	int found=0;
	int test=0;
	for(i=1;i<=20;i++) {
		portnum=0;
		sprintf(adapter, "{%i,5}\0",i);
		do {
		if((portnum = owAcquireEx(adapter)) < 0) {
			printf("ERROR com%i\n",i);
		} else {
			found=1;
			printf("ow found on com%i\n",i);
			rslt = owFirst(portnum, TRUE, FALSE);
			cnt=0;
	        while (rslt) {
				cnt++;
				owSerialNum(portnum,&SNum[0],TRUE);
    			printf("%02X%02X%02X%02X%02X%02X%02X%02X\n",SNum[7],SNum[6],SNum[5],SNum[4],SNum[3],SNum[2],SNum[1],SNum[0]);
				switch (SNum[0]) {
				case 0xA2:
					if (test==0)  {test_normal_mode(portnum,&SNum[0]); test++;}
						else test_continuous_mode(portnum,&SNum[0]);
					break;

				}
				rslt = owNext(portnum, TRUE, FALSE);



			}
			owRelease(portnum);
		}

		} while (found);
	}
	scanf("%i",&i);
	return 0;
}